{ -*- buffer-read-only: t -*- }
{ DON'T EDIT -- this file was automatically generated from "welcome_scene.wrl" }
'#VRML V2.0 utf8' + LineEnding +
'' + LineEnding +
'Group {' + LineEnding +
'  children [' + LineEnding +
'    # TODO: In the future I plan to add here some nice "welcome scene"' + LineEnding +
'    # Text { string [ "Welcome to", "view3dscene" ] }' + LineEnding +
'    WorldInfo { title "Welcome scene" }' + LineEnding +
'  ]' + LineEnding +
'}' + LineEnding +
''
